@@ -7,185 +7,6 @@ using Mono.Unix;
 
 namespace Tomboy
 {
-	public class NoteEditor : Gtk.TextView
-	{
-		public NoteEditor (Gtk.TextBuffer buffer)
-			: base (buffer)
-		{
-			WrapMode = Gtk.WrapMode.Word;
-			LeftMargin = DefaultMargin;
-			RightMargin = DefaultMargin;
-			CanDefault = true;
-
-			// Make sure the cursor position is visible
-			ScrollMarkOnscreen (buffer.InsertMark);
-
-			// Set Font from GConf preference
-			if ((bool) Preferences.Get (Preferences.ENABLE_CUSTOM_FONT)) {
-				string font_string = (string) 
-					Preferences.Get (Preferences.CUSTOM_FONT_FACE);
-				ModifyFont (Pango.FontDescription.FromString (font_string));
-			}
-			Preferences.SettingChanged += OnFontSettingChanged;
-
-			// Set extra editor drag targets supported (in addition
-			// to the default TextView's various text formats)...
-			Gtk.TargetList list = Gtk.Drag.DestGetTargetList (this);
-			list.Add (Gdk.Atom.Intern ("text/uri-list", false), 0, 1);
-			list.Add (Gdk.Atom.Intern ("_NETSCAPE_URL", false), 0, 1);
-			
-			KeyPressEvent += KeyPressed;
-			ButtonPressEvent += ButtonPressed;
-		}
-
-		public static int DefaultMargin
-		{
-			get { return 8; }
-		}
-
-		//
-		// Update the font based on the changed Preference dialog setting.
-		//
-		void OnFontSettingChanged (object sender, GConf.NotifyEventArgs args)
-		{
-			switch (args.Key) {
-			case Preferences.ENABLE_CUSTOM_FONT:
-				Logger.Log ("Switching note font {0}...", 
-					    (bool) args.Value ? "ON" : "OFF");
-
-				if ((bool) args.Value) {
-					string font_string = (string) 
-						Preferences.Get (Preferences.CUSTOM_FONT_FACE);
-					ModifyFont (Pango.FontDescription.FromString (font_string));
-				} else
-					ModifyFont (new Pango.FontDescription ());
-
-				break;
-
-			case Preferences.CUSTOM_FONT_FACE:
-				Logger.Log ("Switching note font to '{0}'...", 
-					    (string) args.Value);
-
-				ModifyFont (Pango.FontDescription.FromString ((string) args.Value));
-				break;
-			}
-		}
-
-		//
-		// DND Drop handling
-		//
-		protected override void OnDragDataReceived (Gdk.DragContext context, 
-							    int x,
-							    int y,
-							    Gtk.SelectionData selection_data,
-							    uint info,
-							    uint time)
-		{
-			bool has_url = false;
-
-			foreach (Gdk.Atom target in context.Targets) {
-				if (target.Name == "text/uri-list" ||
-				    target.Name == "_NETSCAPE_URL") {
-					has_url = true;
-					break;
-				}
-			}
-
-			if (has_url) {
-				UriList uri_list = new UriList (selection_data);
-				bool more_than_one = false;
-				
-				// Place the cursor in the position where the uri was
-				// dropped, adjusting x,y by the TextView's VisibleRect.
-				Gdk.Rectangle rect = VisibleRect;
-				int adjustedX = x + rect.X;
-				int adjustedY = y + rect.Y;
-				Gtk.TextIter cursor = GetIterAtLocation (adjustedX, adjustedY);
-				Buffer.PlaceCursor (cursor);
-				
-				Gtk.TextTag link_tag = Buffer.TagTable.Lookup ("link:url");
-
-				foreach (Uri uri in uri_list) {
-					Logger.Log ("Got Dropped URI: {0}", uri);
-					string insert;
-					if (uri.IsFile)
-						insert = uri.LocalPath;
-					else
-						insert = uri.ToString ();
-					
-					if (insert == null || insert.Trim () == String.Empty)
-						continue;
-					
-					if (more_than_one) {
-						cursor = Buffer.GetIterAtMark (Buffer.InsertMark);
-						
-						// FIXME: The space here is a hack
-						// around a bug in the URL Regex which
-						// matches across newlines.
-						if (cursor.LineOffset == 0)
-							Buffer.Insert (ref cursor, " \n");
-						else
-							Buffer.Insert (ref cursor, ", ");
-					}
-					
-					Buffer.InsertWithTags (ref cursor, insert, link_tag);
-					more_than_one = true;
-				}
-
-				Gtk.Drag.Finish (context, more_than_one, false, time);
-			} else {
-				base.OnDragDataReceived (context, x, y, selection_data, info, time);
-			}
-		}
-		
-		[GLib.ConnectBefore()]
-		void KeyPressed (object sender, Gtk.KeyPressEventArgs args)
-		{
-			args.RetVal = true;
-			bool ret_value = false;
-			
-			switch (args.Event.Key)
-			{
-			case Gdk.Key.Return:
-				ret_value = ((NoteBuffer) Buffer).AddNewline ();
-				ScrollMarkOnscreen (Buffer.InsertMark);
-				break;
-			case Gdk.Key.Tab:
-				ret_value = ((NoteBuffer) Buffer).AddTab ();
-				ScrollMarkOnscreen (Buffer.InsertMark);
-				break;
-			case Gdk.Key.ISO_Left_Tab:
-				ret_value = ((NoteBuffer) Buffer).RemoveTab ();
-				ScrollMarkOnscreen (Buffer.InsertMark);
-				break;
-			case Gdk.Key.Delete:
-				ret_value = ((NoteBuffer) Buffer).DeleteKeyHandler ();
-				ScrollMarkOnscreen (Buffer.InsertMark);
-				break;
-			case Gdk.Key.BackSpace:
-				ret_value = ((NoteBuffer) Buffer).BackspaceKeyHandler ();
-				break;
- 			case Gdk.Key.Left:
- 			case Gdk.Key.Right:
- 			case Gdk.Key.Up:
- 			case Gdk.Key.Down:
-				ret_value = false;
-				break;
-			default:
-				((NoteBuffer) Buffer).CheckSelection ();
-				break;
-			}
-			
-			args.RetVal = ret_value;
-		}
-		
-		[GLib.ConnectBefore()]
-		void ButtonPressed (object sender, Gtk.ButtonPressEventArgs args)
-		{
-			((NoteBuffer) Buffer).CheckSelection ();
-		}
-	}
-
 	public class NoteWindow : ForcedPresentWindow 
 	{
 		Note note;
@@ -616,13 +437,14 @@ namespace Tomboy
 					      Catalog.GetString ("Set properties of text"), 
 					      null);
 			
-			// FIXME: Use a proper icon for the Tags button
+			// FIXME: Isn't there a better way to load an icon besides having
+			// to specify a size implicitly?  (tomboy-tag, 22)
 			Gtk.Widget tags_button =
 				toolbar.AppendItem (
 					Catalog.GetString ("Tags"),
 					Catalog.GetString ("Add/Remove note tags"),
 					null,
-					new Gtk.Image (Gtk.Stock.SelectFont, toolbar.IconSize),
+					new Gtk.Image (GuiUtils.GetIcon ("tomboy-tag", 22)),
 					new Gtk.SignalFunc (TagBarClicked));
 			tags_button.AddAccelerator ("activate",
 						    accel_group,
