@@ -341,7 +341,8 @@ namespace Tomboy
 		public UriList (Gtk.SelectionData selection) 
 		{
 			// FIXME this should check the atom etc.
-			LoadFromString (Encoding.UTF8.GetString (selection.Data));
+			if (selection.Length > 0)
+				LoadFromString (Encoding.UTF8.GetString (selection.Data));
 		}
 
 		public override string ToString () 
@@ -797,6 +798,10 @@ namespace Tomboy
 			action_manager.LoadInterface ();
 
 			RegisterSignalHandlers ();
+			
+			// Register handler for saving session when logging out of Gnome
+			Gnome.Client client = Gnome.Global.MasterClient ();
+			client.SaveYourself += OnSaveYourself;
 		}
 
 		[DllImport("libc")]
@@ -827,6 +832,14 @@ namespace Tomboy
 			Stdlib.signal (Signum.SIGINT, OnExitSignal);
 		}
 
+		static void OnSaveYourself (object sender, Gnome.SaveYourselfArgs args)
+		{
+			Logger.Log ("Received request for saving session");
+			
+			if (ExitingEvent != null)
+				ExitingEvent (null, new EventArgs ());
+		}
+
 		public static void RegisterSessionManagerRestart (string executable_path,
 								  string[] args, 
 								  string[] environment)
